@@ -3,27 +3,27 @@
  * ========================================================================
  *
  * (C) Copyright 2000-2005, by Object Refinery Limited and Contributors.
- * 
+ *
  * Project Info:  http://www.jfree.org/jcommon/index.html
  *
- * This library is free software; you can redistribute it and/or modify it 
- * under the terms of the GNU Lesser General Public License as published by 
- * the Free Software Foundation; either version 2.1 of the License, or 
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, but 
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
  * License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
- * USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+ * USA.
  *
- * [Java is a trademark or registered trademark of Sun Microsystems, Inc. 
+ * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
  * in the United States and other countries.]
- * 
+ *
  * -------------------
  * AbstractModule.java
  * -------------------
@@ -32,7 +32,7 @@
  * Original Author:  Thomas Morgner;
  * Contributor(s):   David Gilbert (for Object Refinery Limited);
  *
- * $Id: AbstractModule.java,v 1.3 2005/10/18 13:14:50 mungady Exp $
+ * $Id: AbstractModule.java,v 1.7 2008/09/10 09:16:54 mungady Exp $
  *
  * Changes
  * -------
@@ -114,7 +114,7 @@ public abstract class AbstractModule extends DefaultModuleInfo implements Module
      *
      * @param reader the buffered reader that is the source of the text.
      */
-    public ReaderHelper(final BufferedReader reader)
+    protected ReaderHelper(final BufferedReader reader)
     {
       this.reader = reader;
     }
@@ -223,6 +223,7 @@ public abstract class AbstractModule extends DefaultModuleInfo implements Module
       throw new ModuleInitializeException
           ("File 'module.properties' not found in module package.");
     }
+
     loadModuleInfo(in);
   }
 
@@ -236,41 +237,46 @@ public abstract class AbstractModule extends DefaultModuleInfo implements Module
    */
   protected void loadModuleInfo(final InputStream in) throws ModuleInitializeException
   {
+    if (in == null)
+    {
+      throw new NullPointerException
+          ("Given InputStream is null.");
+    }
+
     try
     {
-      if (in == null)
-      {
-        throw new NullPointerException
-            ("Given InputStream is null.");
-      }
-      final ReaderHelper rh = new ReaderHelper(new BufferedReader
-          (new InputStreamReader(in, "ISO-8859-1")));
-
       final ArrayList optionalModules = new ArrayList();
       final ArrayList dependendModules = new ArrayList();
-
-      while (rh.hasNext())
+      final ReaderHelper rh = new ReaderHelper(new BufferedReader
+          (new InputStreamReader(in, "ISO-8859-1")));
+      try
       {
-        final String lastLineRead = rh.next();
-        if (lastLineRead.startsWith("module-info:"))
+        while (rh.hasNext())
         {
-          readModuleInfo(rh);
-        }
-        else if (lastLineRead.startsWith("depends:"))
-        {
-          dependendModules.add(readExternalModule(rh));
-        }
-        else if (lastLineRead.startsWith("optional:"))
-        {
-          optionalModules.add(readExternalModule(rh));
-        }
-        else
-        {
-          // we dont understand the current line, so we skip it ...
-          // should we throw a parse exception instead?
+          final String lastLineRead = rh.next();
+          if (lastLineRead.startsWith("module-info:"))
+          {
+            readModuleInfo(rh);
+          }
+          else if (lastLineRead.startsWith("depends:"))
+          {
+            dependendModules.add(readExternalModule(rh));
+          }
+          else if (lastLineRead.startsWith("optional:"))
+          {
+            optionalModules.add(readExternalModule(rh));
+          }
+          else
+          {
+            // we dont understand the current line, so we skip it ...
+            // should we throw a parse exception instead?
+          }
         }
       }
-      rh.close();
+      finally
+      {
+        rh.close();
+      }
 
       this.optionalModules = (ModuleInfo[])
           optionalModules.toArray(new ModuleInfo[optionalModules.size()]);
@@ -321,7 +327,7 @@ public abstract class AbstractModule extends DefaultModuleInfo implements Module
 
   /**
    * Checks, whether the next line in the reader is a value line.
-   * 
+   *
    * @param reader from where to read the lines.
    * @return true, if the next line is a value line, false otherwise.
    * @throws IOException if an IO error occurs.
@@ -373,31 +379,31 @@ public abstract class AbstractModule extends DefaultModuleInfo implements Module
         // parse error: Non data line does not contain a colon
         final String b = readValue(reader, parseValue(line.trim()));
 
-        if (key.equals("name"))
+        if ("name".equals(key))
         {
           setName(b);
         }
-        else if (key.equals("producer"))
+        else if ("producer".equals(key))
         {
           setProducer(b);
         }
-        else if (key.equals("description"))
+        else if ("description".equals(key))
         {
           setDescription(b);
         }
-        else if (key.equals("subsystem"))
+        else if ("subsystem".equals(key))
         {
           setSubSystem(b);
         }
-        else if (key.equals("version.major"))
+        else if ("version.major".equals(key))
         {
           setMajorVersion(b);
         }
-        else if (key.equals("version.minor"))
+        else if ("version.minor".equals(key))
         {
           setMinorVersion(b);
         }
-        else if (key.equals("version.patchlevel"))
+        else if ("version.patchlevel".equals(key))
         {
           setPatchLevel(b);
         }
@@ -471,19 +477,19 @@ public abstract class AbstractModule extends DefaultModuleInfo implements Module
       if (key != null)
       {
         final String b = readValue(reader, parseValue(line));
-        if (key.equals("module"))
+        if ("module".equals(key))
         {
           mi.setModuleClass(b);
         }
-        else if (key.equals("version.major"))
+        else if ("version.major".equals(key))
         {
           mi.setMajorVersion(b);
         }
-        else if (key.equals("version.minor"))
+        else if ("version.minor".equals(key))
         {
           mi.setMinorVersion(b);
         }
-        else if (key.equals("version.patchlevel"))
+        else if ("version.patchlevel".equals(key))
         {
           mi.setPatchLevel(b);
         }
@@ -644,12 +650,40 @@ public abstract class AbstractModule extends DefaultModuleInfo implements Module
    *
    * @param name the name of the library class.
    * @return true, if the class could be loaded, false otherwise.
+   * @deprecated use the method that passes in a context-class.
    */
   protected static boolean isClassLoadable(final String name)
   {
     try
     {
-      Thread.currentThread().getContextClassLoader().loadClass(name);
+      final ClassLoader loader = ObjectUtilities.getClassLoader(AbstractModule.class);
+      if (loader == null)
+      {
+        // this should not happen .. If it happens, it measn we dont even have a system-classloader.
+        return false;
+      }
+      loader.loadClass(name);
+      return true;
+    }
+    catch (Exception e)
+    {
+      return false;
+    }
+  }
+
+  /**
+   * Tries to load a class to indirectly check for the existence
+   * of a certain library.
+   *
+   * @param name the name of the library class.
+   * @param context the context class to get a classloader from.
+   * @return true, if the class could be loaded, false otherwise.
+   */
+  protected static boolean isClassLoadable(final String name, final Class context)
+  {
+    try
+    {
+      ObjectUtilities.getClassLoader(context).loadClass(name);
       return true;
     }
     catch (Exception e)
@@ -661,7 +695,7 @@ public abstract class AbstractModule extends DefaultModuleInfo implements Module
   /**
    * Configures the module by loading the configuration properties and
    * adding them to the package configuration.
-   * 
+   *
    * @param subSystem  the subsystem.
    */
   public void configure(final SubSystem subSystem)
@@ -672,7 +706,21 @@ public abstract class AbstractModule extends DefaultModuleInfo implements Module
     {
       return;
     }
-    subSystem.getPackageManager().getPackageConfiguration().load(in);
+    try
+    {
+      subSystem.getPackageManager().getPackageConfiguration().load(in);
+    }
+    finally
+    {
+      try
+      {
+        in.close();
+      }
+      catch (IOException e)
+      {
+        // can be ignored ...
+      }
+    }
   }
 
   /**
@@ -681,14 +729,49 @@ public abstract class AbstractModule extends DefaultModuleInfo implements Module
    *
    * @param classname the class name of the initializer.
    * @throws ModuleInitializeException if an error occures
+   * @deprecated Use the method that provides a class-context instead.
    */
   protected void performExternalInitialize(final String classname)
       throws ModuleInitializeException
   {
     try
     {
-      final Class c = Thread.currentThread().getContextClassLoader().loadClass(classname);
-      final ModuleInitializer mi = (ModuleInitializer) c.newInstance();
+      final ModuleInitializer mi =
+          (ModuleInitializer) ObjectUtilities.loadAndInstantiate(classname, AbstractModule.class, ModuleInitializer.class);
+      if (mi == null)
+      {
+        throw new ModuleInitializeException("Failed to load specified initializer class.");
+      }
+      mi.performInit();
+    }
+    catch (ModuleInitializeException mie)
+    {
+      throw mie;
+    }
+    catch (Exception e)
+    {
+      throw new ModuleInitializeException("Failed to load specified initializer class.", e);
+    }
+  }
+
+  /**
+   * ???.
+   *
+   * @param classname ?
+   * @param context ?
+   * @throws ModuleInitializeException
+   */
+  protected void performExternalInitialize(final String classname, final Class context)
+      throws ModuleInitializeException
+  {
+    try
+    {
+      final ModuleInitializer mi =
+          (ModuleInitializer) ObjectUtilities.loadAndInstantiate(classname, context, ModuleInitializer.class);
+      if (mi == null)
+      {
+        throw new ModuleInitializeException("Failed to load specified initializer class.");
+      }
       mi.performInit();
     }
     catch (ModuleInitializeException mie)
